!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_modules-user-arrays.f90
!>
!> \brief User-defined module example
!>
!> This file is compiled before all other user Fortran files.
!> To ensure this, it must not be renamed.
!>
!> The user may define an arbitrary number of modules here, even though
!> only one is defined in the example.
!
!> \cond DOXYGEN_SHOULD_SKIP_THIS

!-------------------------------------------------------------------------------

module Physical_properties_table

use entsor
use parall

implicit none    
logical :: table_dens_read = .false.
!!! ALLOCATABLE TARGET FOE DENSITY VALUES !!!
double precision, dimension(:,:), allocatable, target :: density_table

    contains
    
    subroutine READ_DATA()
        implicit none 
        
        character(1) :: dummy
        double precision :: temp
        double precision :: Dens
        integer :: nlines
        integer :: i
        integer :: io
        
        !! IF FILENAMES ALTER REPLACE HERE !!    
        character(50) :: filename = "CO2_30BAR_100-2000.txt"
        
        !!! IF TABLE DENS HAS BEEN ASSIGNED RETURN !!!
        if (table_dens_read) return

        !!! OPEN THE FILE TO BE READ !!!
        open(798, FILE=filename, STATUS ='OLD')
        
        !!! CHECK THE NUMBER OF LINES IN FILE !!!
        nlines = 0 
        do 
          read(798, *, iostat = io) dummy, dummy, dummy, dummy, dummy, dummy
          if (io/=0) exit
          nlines = nlines + 1 
        end do 
        
        rewind(798)
        
        !!!JUST IN CASE CALCULATE THE MAX OVER ALL PROCESSES !!!
        if (irangp .ge. 0 ) then
           call parcmx(nlines)
           write(nfecra, *) "NUMBER OF LINES", nlines
        end if
        allocate(density_table(2, nlines-1))

        !!! INITIALISE TO ZERO !!!        
        do i = 1, nlines-1
           density_table(1, i) = 0d0
           density_table(2, i) = 0d0
        end do    

        !!! READING HEADINGS !!!
        read(798, *) dummy, dummy, dummy, dummy, dummy, dummy
        do i = 1, nlines-1
           read(798, *) temp, dummy, Dens, dummy, dummy, dummy
           density_table(1, i) = temp 
           density_table(2, i) = Dens 
           !write(*,*) temp, Dens 
        end do

        !!! CLOSE THE FILE !!! 
        close(798)
        
        
        !!! PERFORM CHECK ON ALLOCATABLE ARRAY FOR STILL INITIALISED !!!
        !!! VALS                                                     !!!
        do i = 1, nlines-1
           if ((density_table(1, i) .eq. 0d0) .or. &
              (density_table(2, i)) .eq. 0d0) then
              write(nfecra, *) "LOCATION OF VAL -->", i 
              call csexit(1) 
           end if  
        end do   
        !!! SET LOGICAL TO TRUE !!!
        table_dens_read = .true. 
    end subroutine READ_DATA

end module Physical_properties_table

!-------------------------------------------------------------------------------

!> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
